### HIPE script to save PACS level-1 data to fits files for input to Scanamorphos
### supplied by Marc Sauvage

### note: Advanced scripts to process PACS data from level 0 to level 1 can be obtained
### from Marc Sauvage.


def convertL1ToScanam(frames,obsid=0l):
    """
    This code converts level1 frames obtained within the context of the classical hipe 
    pipeline into level1 frames that can be saved to fits and ingested in Scanamorphos.
    
    convertL1ToScanam(frames,obsid=obsid)
    
    obsid is an optional parameter that you can use to provide the obsid if it is not
    in the metadata.

    The frames object is saved in the current directory under the name:
        [obsid]_[filter]_Scanam_level1Frames.fits
    """
    # HISTORY
    # 29-Jul-2010 MS Creation
    # Import section
    from herschel.ia.all import *
    from java.util import Date
    
    # Scanamorphos needs three masks to be present in the frame structure.
    # It is intelligent enough for the masks to be in any order, but they
    # have to have the following names:
    # SA_badpix  for all bad pixels
    # NonScience for all the preamble data that occur before the scan
    # UNCLEANCHOP that contains a mask for the drifts occurring after calibration blocks.
    print "Creating the scanamorphos bad pixel mask and converting mask to boolean arrays", Date()
    # - badpixels is the or of BADPIXELS, BLINDPIXELS, SATURATION, MMT_Glitchmask, SecondGlitchmask
    origList = ['BADPIXELS','BLINDPIXELS','SATURATION','MMT_Glitchmask','SecondGlitchmask']
    nMasks = len(origList)
    maskList = String1d()
    for i in range(nMasks):
        if (frames.mask.containsMask(origList[i])):
            maskList.append(origList[i])
    # now merge these masks and place them in the SA_badpix mask
    frames.mask.merge(maskList,'SA_badpix')
    # now check to see if the other two masks are present.
    if (frames.mask.containsMask('NonScience') == False):
        print "WARNING -- The NonScience mask is absent"
        print "WARNING -- It flags preamble data occuring before the first leg"
        print "WARNING -- Scanamorphos will not crash but artifacts may be present"
    if (frames.mask.containsMask('UNCLEANCHOP') == False):
        print "WARNING -- The UNCLEANCHOP mask is absent"
        print "WARNING -- (not necessary if you do not intend to mask transients)"
    
    print "Converting masks to boolean array that scanamorphos can read"
    frames.mask.convert2boolean()
    
    print "Saving level 1 frames converted for scanamorphos", Date()
    # I need some meta-data information to make up a name so still modifications to be done here.
    # check to see if the user has not specified the obsid
    if (obsid == 0l):
        if (frames.meta.containsKey('obsid')):
            obsid = frames.meta['obsid'].value
        else:
            print "WARNING -- obsid not found in the frames metadata - maybe they are old"
            obsid=0l
    
    if (frames.meta.containsKey('camName') == False):
        print "ERROR -- the key camName is not in the metadata"
        print "ERROR -- As this tells which side (blue/red) is present in the frames,"
        print "ERROR -- this is a really worrying sign regarding the data."
        print "ERROR -- Please investigate before trying scanamorphos"
        return
    
    if (frames.meta['camName'].value == 'Red Photometer'):
        side = 'red'
    else:
        if (frames.meta['blue'].value == 'blue1'):
            side = 'blue'
        else:
            side = 'green'
    FitsArchive().save(str(obsid)+'_'+side+'_Scanam_level1Frames.fits',frames)
